/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is Calendar Palm Sync code.
 *
 * The Initial Developer of the Original Code is
 * Michiel van Leeuwen (mvl@exedo.nl).
 * Portions created by the Initial Developer are Copyright (C) 2004
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */

/**
 * This is just a fake sync client, to test the algorithm.
 * it is easier to change the records in the code here then do work
 * on the palm. And more important, to have reproducable stuff
 */

function nsDummySyncRecord(aRecord) { 
  this.mTitle = aRecord[0];
  this.mId = aRecord[1];
  this.mDirty = aRecord[2];
  this.mDeleted = false;
}

nsDummySyncRecord.prototype = {
  get id()       { return this.mId; },
  get deleted()  { return this.mDeleted; },
  get dirty()    { return this.mDirty; },
  get title()    { return this.mTitle; },

  get category() {
    throw Components.results.NS_ERROR_NOT_IMPLEMENTED;
  },

  get localTodo() { return this.mLocalTodo; },

  QueryInterface: function (iid) {
    if (!iid.equals(Components.interfaces.nsIDummySyncRecord) &&
        !iid.equals(Components.interfaces.nsISyncRecord) &&
        !iid.equals(Components.interfaces.nsISupports)) {
        throw Components.results.NS_ERROR_NO_INTERFACE;
    }
    return this;
  },
  
  mId: null,
  mDeleted: null,
  mDirty: null,
  mTitle: null
}


function nsDummyEnumerator(aRecords) {
  this.mRecords = aRecords;
  this.mIndex = 0;
}

nsDummyEnumerator.prototype = {
  hasMoreElements: function() {
    return (this.mIndex < this.mRecords.length);
  },

  getNext: function() {
    return new nsDummySyncRecord(this.mRecords[this.mIndex++]);
  },
  
  mRecords: null,
  mIndex: null
}

function nsDummySync() { }

nsDummySync.prototype = {

  get identifier() { return "DummySync"; },

  init: function() {
    // title, id, dirty
    this.mLocalRecords  = [["Rec1",1,1],["Rec2",2,0],["Extr",3,0]];
    this.mRemoteRecords = [["Fooo",1,1],["Barr",2,1],["Extr",3,0]];
    
    this.dump();
  },
  
  finish: function() {
    this.dump();
  },

  get localRecords() {
    return new nsDummyEnumerator(this.mLocalRecords);
  },

  get remoteRecords() {
    return new nsDummyEnumerator(this.mRemoteRecords);
  },

  getLocalRecordBySyncId: function(aId) {
    for (var i = 0; i < this.mLocalRecords.length; ++i) {
      if (this.mLocalRecords[i][1] == aId) {
        return new nsDummySyncRecord(this.mLocalRecords[i]);
      }
    }
    return null;
  },

  getRemoteRecordBySyncId: function(aId) {
    for (var i = 0; i < this.mRemoteRecords.length; ++i) {
      if (this.mRemoteRecords[i][1] == aId) {
        return new nsDummySyncRecord(this.mRemoteRecords[i]);
      }
    }
    return null;
  },

  addLocal: function (aRecord) {
    var remoteRecord = aRecord.QueryInterface(Components.interfaces.nsIDummySyncRecord);
dump("addLocal "+remoteRecord.title+"\n");
    this.mLocalRecords.push([remoteRecord.title, remoteRecord.id, 0]);
  },

  addRemote: function (aRecord) {
    var localRecord = aRecord.QueryInterface(Components.interfaces.nsIDummySyncRecord);
dump("addRemote "+localRecord.title+"\n");
    this.mRemoteRecords.push([localRecord.title, localRecord.id, 0]);
  },

  removeLocal: function (aRecord) {
    var localRecord = aRecord.QueryInterface(Components.interfaces.nsIDummySyncRecord);
dump("removeLocal "+localRecord.title+"\n");
    for (var i = 0; i < this.mLocalRecords.length; ++i) {
      if (this.mLocalRecords[i][1] == localRecord.id)
        this.mLocalRecords.splice(i, 1);
    }
    return null;
  },

  removeRemote: function (aRecord) {
    var remoteRecord = aRecord.QueryInterface(Components.interfaces.nsIDummySyncRecord);
dump("removeRemote "+remoteRecord.title+"\n");
    for (var i = 0; i < this.mRemoteRecords.length; ++i) {
      if (this.mRemoteRecords[i][1] == remoteRecord.id)
        this.mRemoteRecords.splice(i, 1);
    }
    return null;
  },

  updateLocal: function (aRemoteRecord, aLocalRecord) {
    var remoteRecord = aRemoteRecord.QueryInterface(Components.interfaces.nsIDummySyncRecord);
    var localRecord = aLocalRecord.QueryInterface(Components.interfaces.nsIDummySyncRecord);
dump("updateLocal "+remoteRecord.title+"  "+aLocalRecord.id+"\n");
    for (var i = 0; i < this.mLocalRecords.length; ++i) {
      if (this.mLocalRecords[i][1] == localRecord.id) {
        this.mLocalRecords[i][0] = remoteRecord.title;
        this.mLocalRecords[i][1] = remoteRecord.id;
      }
    }
    return null;
  },

  updateRemote: function (aLocalRecord, aRemoteRecord) {
    var localRecord = aLocalRecord.QueryInterface(Components.interfaces.nsIDummySyncRecord);
    var remoteRecord = aRemoteRecord.QueryInterface(Components.interfaces.nsIDummySyncRecord);
dump("updateRemote "+localRecord.title+"  "+aRemoteRecord.id+"\n");
    for (var i = 0; i < this.mRemoteRecords.length; ++i) {
      if (this.mRemoteRecords[i][1] == remoteRecord.id) {
        this.mRemoteRecords[i][0] = localRecord.title;
        this.mRemoteRecords[i][1] = localRecord.id;
      }
    }
    return null;
  },

  compare: function (aLocalRecord, aRemoteRecord) {
    var localRecord = aLocalRecord.QueryInterface(Components.interfaces.nsIDummySyncRecord);
    var remoteRecord = aRemoteRecord.QueryInterface(Components.interfaces.nsIDummySyncRecord);
dump("compare "+localRecord.title+" to "+remoteRecord.title+"\n");
    if (localRecord.title != remoteRecord.title)
      return false;
    if (localRecord.id != remoteRecord.id)
      return false;
    // XXX due date
dump("same!\n");
    return true;
  },

  dump: function() {
    dump("Local: \n");
    for (var i = 0; i < this.mLocalRecords.length; ++i) {
      dump(" ID: "+this.mLocalRecords[i][1]);
      dump(" Title: "+this.mLocalRecords[i][0]+"\n");
    }
    dump("Remote: \n");
    for (i = 0; i < this.mRemoteRecords.length; ++i) {
      dump(" ID: "+this.mRemoteRecords[i][1]);
      dump(" Title: "+this.mRemoteRecords[i][0]+"\n");
    }
  },

  QueryInterface: function (iid) {
    if (!iid.equals(Components.interfaces.nsISyncClient) &&
        !iid.equals(Components.interfaces.nsISupports)) {
        throw Components.results.NS_ERROR_NO_INTERFACE;
    }
    return this;
  },

  mLocalRecords : null,
  mRemoteRecords : null
}

var syncModule = {
  firstTime: true,

  registerSelf: function (compMgr, fileSpec, location, type) {
    if (this.firstTime) {
      this.firstTime = false;
      throw Components.results.NS_ERROR_FACTORY_REGISTER_AGAIN;
    }
    compMgr = compMgr.QueryInterface(Components.interfaces.nsIComponentRegistrar);
    compMgr.registerFactoryLocation(this.dummySyncCID,
                                    "Dummy Sync JS Component",
                                    this.dummySyncProgID, 
                                    fileSpec,
                                    location,
                                    type);
  },

  getClassObject: function (compMgr, cid, iid) {
    if (cid.equals(this.dummySyncCID))
      return this.dummySyncFactory;

    if (!iid.equals(Components.interfaces.nsIFactory))
      throw Components.results.NS_ERROR_NOT_IMPLEMENTED;

    throw Components.results.NS_ERROR_NO_INTERFACE;
  },

  dummySyncCID: Components.ID("{713b42df-2463-43b9-8eb8-57dfc5d44805}"),
  dummySyncProgID: "@mozilla.org/sync/dummy;1",
  dummySyncFactory: {
    createInstance: function (outer, iid) {
      if (outer != null)
        throw Components.results.NS_ERROR_NO_AGGREGATION;

      var instance = new nsDummySync();
      instance.init();
      return instance.QueryInterface(iid);
    }
  },

  canUnload: function(compMgr) {
    return true;
  }
};

function NSGetModule(compMgr, fileSpec) {
  return syncModule;
}
