/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is Calendar Palm Sync code.
 *
 * The Initial Developer of the Original Code is
 * Michiel van Leeuwen (mvl@exedo.nl).
 * Portions created by the Initial Developer are Copyright (C) 2004
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */

function nsLocalTodoSyncRecord(aId, aDeleted, aCategory, aLocalEvent, aLastSync) { 
  this.mId = aId;
  this.mDeleted = aDeleted;
  this.mLocalEvent = aLocalEvent;

  this.mDirty = false;
  try {
    if ((nextLocalEvent.lastModified / 1000) > aLastSync)
      this.mDirty = true;
  } catch(e) {
    // Looks like the record wasn't modified at all. assume it is new
    this.mDirty = true
  }
}

nsLocalTodoSyncRecord.prototype = {

  get id()       { return this.mId; },
  get deleted()  { return this.mDeleted; },
  get dirty()    { return this.mDirty; },

  get category() {
    throw Components.results.NS_ERROR_NOT_IMPLEMENTED;
  },

  get localTodo() { return this.mLocalTodo; },

  QueryInterface: function (iid) {
    if (!iid.equals(Components.interfaces.nsILocalTodoSyncRecord) &&
        !iid.equals(Components.interfaces.nsISyncRecord) &&
        !iid.equals(Components.interfaces.nsISupports)) {
        throw Components.results.NS_ERROR_NO_INTERFACE;
    }
    return this;
  },
  
  mId: null,
  mDeleted: null,
  mDirty: null,
  mLocalTodo: null
}


const nsIPalmConnection = Components.interfaces.nsIPalmConnection;

function nsPalmTodoEnumerator(aPalmConnection) {
  this.mPalmConnection = aPalmConnection;
  this.prefetch();
}

nsPalmTodoEnumerator.prototype = {

  hasMoreElements: function() {
    return (this.mNext != null);
  },

  getNext: function() {
    var next = this.mNext;
    this.mIndex++;
    this.prefetch();
    return next;
  },
  
  prefetch: function() {
    try {
      this.mNext = this.mPalmConnection.readRecordByIndex(this.mIndex);
      this.mNext = this.mNext.QueryInterface(Components.interfaces.nsIPalmTodoSyncRecord);
    } catch(e) {
      this.mNext = null;
    }
  },
  
  mPalmConnection: null,
  mNext: null,
  mIndex: null
}

function nsLocalTodoEnumerator(aTodoEnum, aLastSync) {
  this.mTodoEnum = aTodoEnum;
  this.mLastSync = aLastSync;
}

nsLocalTodoEnumerator.prototype = {
  hasMoreElements: function() {
    return (this.mTodoEnum.hasMoreElements());
  },

  getNext: function() {
    var nextLocalTodo = this.mTodoEnum.getNext().QueryInterface(Components.interfaces.oeIICalTodo);
    var dirty = false;
    try {
      if ((nextLocalTodo.lastModified / 1000) > this.mLastSync)
        dirty = true;
    } catch(e) {
      // Looks like the record wasn't modified at all. assume it is new
      dirty = true
    }
    return new nsLocalTodoSyncRecord(nextLocalTodo.syncId,
                                     false, 0,
                                     nextLocalTodo, this.mLastSync);
  },
  
  mTodoEnum: null,
  mLastSync: 0
}


function nsPalmTodoSync() { }

nsPalmTodoSync.prototype = {

  get identifier() { return "PalmTodoSync"; },

  get fastPossible() { return this.mFastPossible; },

  init: function () {
    this.mPalmConnection = Components.classes["@mozilla.org/sync/palmconnection;1"]
                             .createInstance(Components.interfaces.nsIPalmConnection);
    this.mPalmConnection.connect();
    this.mPalmConnection.openDataBase(nsIPalmConnection.PALM_TODO);

    var prefs = Components.classes["@mozilla.org/preferences-service;1"].
                  getService(Components.interfaces.nsIPrefService);
    var prefbranch = prefs.getBranch(null);

    try {
      this.mCalendarPath = prefbranch.getCharPref("calendar.sync.server");
    } catch(e) {}
    dump("path: "+this.mCalendarPath+"\n");

    var icalContainer = Components.classes["@mozilla.org/ical-container;1"]
                           .getService(Components.interfaces.oeIICalContainer);
    this.mCalendar = icalContainer.getCalendar(this.mCalendarPath);

    dump("userID:       "+this.mPalmConnection.userID+"\n");
    dump("viewerID:     "+this.mPalmConnection.viewerID+"\n");

    this.mLastSync = 0;
    try {
      this.mLastSync = prefbranch.getIntPref("calendar.sync.lastsync.PalmTodoSync");
    } catch(e) {}

    var pcId = 0;
    try {
      pcId = prefbranch.getIntPref("calendar.sync.palm.pcid");
    } catch(e) {}


    this.mFastPossible = false;
    // XXX is only lastSyncPC engouh?
    if ((this.mLastSync == this.mPalmConnection.successfulSyncDate) &&
        (pcId == this.mPalmConnection.lastSyncPC))
      this.mFastPossible = true;

    dump("localLastSync:"+this.mLastSync+"\n");
    dump("lastSync:     "+this.mPalmConnection.lastSyncDate+"\n");
    dump("pcID:         "+pcId+"\n");
    dump("lastSyncPC:   "+this.mPalmConnection.lastSyncPC+"\n");
    dump("FastPossible: "+this.mFastPossible+"\n");
  },

  finish: function() {
    var prefs = Components.classes["@mozilla.org/preferences-service;1"].
                  getService(Components.interfaces.nsIPrefService);
    var prefbranch = prefs.getBranch(null);
    var now = new Date();
    prefbranch.setIntPref("calendar.sync.lastsync.PalmTodoSync", now.getTime() / 1000);

    var pcId = 0;
    try {
      pcId = prefbranch.getIntPref("calendar.sync.palm.pcid");
    } catch(e) {}

    if (!pcId) {
      pcId = Math.round(Math.random()*1000000);
      prefbranch.setIntPref("calendar.sync.palm.pcid", pcId);
    }

    this.mPalmConnection.lastSyncPC = pcId;
    this.mPalmConnection.successfulSyncDate = now.getTime() / 1000;
    this.mPalmConnection.lastSyncDate = now.getTime() / 1000;
    this.mPalmConnection.writeUserInfo();

    this.mPalmConnection.closeDataBase();
    this.mPalmConnection.disconnect();
  },


  get localRecords() {
    return new nsLocalTodoEnumerator(this.mCalendar.getAllTodos(), this.mLastSync);
  },

  get remoteRecords() {
    return new nsPalmTodoEnumerator(this.mPalmConnection);
  },

  getLocalRecordBySyncId: function(aId) {
    var allTodos = this.mCalendar.getAllTodos();
    while (allTodos.hasMoreElements()) {
      var record = allTodos.getNext().QueryInterface(Components.interfaces.oeIICalEvent);
      if (record.syncId && (record.syncId == aId)) {
        return new nsLocalTodoSyncRecord(record.syncId, false, 0,
                                         record, this.mLastSync);
      }
    }
    return null;
  },

  getRemoteRecordBySyncId: function(aId) {
    return this.mPalmConnection.readRecordById(aId);
  },

  addLocal: function (aRecord) {
    var localTodo = Components.classes["@mozilla.org/icaltodo;1"]
                       .createInstance(Components.interfaces.oeIICalTodo);
    this.fillLocalRecord(aRecord, localTodo);
    var idstr = this.mCalendar.addTodo(localTodo);
    dump("Added "+idstr+"\n");
  },

  addRemote: function (aRecord) {
dump("addRemote "+aRecord.id+"\n");
    var remoteRecord = this.mPalmConnection.newRecord()
                         .QueryInterface(Components.interfaces.nsIPalmTodoSyncRecord);

    this.fillRemoteRecord(aRecord, remoteRecord);
    this.mPalmConnection.writeRecord(remoteRecord);

    var localTodo = aRecord.QueryInterface(Components.interfaces.nsILocalTodoSyncRecord).localTodo;
    localTodo.syncId = remoteRecord.id;
    this.mCalendar.modifyTodo(localTodo);
dump("Tried to add remote "+remoteRecord.description+" to id "+remoteRecord.id+"\n");
  },

  removeLocal: function (aRecord) {
    var localTodo = aRecord.QueryInterface(Components.interfaces.nsILocalTodoSyncRecord).localTodo;
dump("trying to delete "+localTodo.id+"\n");
    this.mCalendar.deleteTodo(localTodo.id);
  },

  removeRemote: function (aRecord) {
    throw Components.results.NS_ERROR_NOT_IMPLEMENTED;
  },

  updateLocal: function (aRemoteRecord, aLocalRecord) {
    var localTodo = aLocalRecord.QueryInterface(Components.interfaces.nsILocalTodoSyncRecord).localTodo;
    this.fillLocalRecord(aRemoteRecord, localTodo);
    this.mCalendar.modifyTodo(localTodo);
  },

  updateRemote: function (aLocalRecord, aRemoteRecord) {
dump("updateRemote "+aRemoteRecord.id+"  "+aLocalRecord.id+"\n");
    var remoteRecord = aRemoteRecord.QueryInterface(Components.interfaces.nsIPalmTodoSyncRecord);
    var localTodo = aLocalRecord.QueryInterface(Components.interfaces.nsILocalTodoSyncRecord).localTodo;

    this.fillRemoteRecord(aLocalRecord, remoteRecord);
    this.mPalmConnection.writeRecord(remoteRecord);

    localTodo.syncId = remoteRecord.id;
    this.mCalendar.modifyTodo(localTodo);
dump("Tried to update remote "+remoteRecord.description+" to id "+remoteRecord.id+"\n");
  },

  compare: function (aLocalRecord, aRemoteRecord) {
    var localTodo = aLocalRecord.QueryInterface(Components.interfaces.nsILocalTodoSyncRecord).localTodo;
    var remoteTodoRecord = aRemoteRecord.QueryInterface(Components.interfaces.nsIPalmTodoSyncRecord);
dump("compare "+localTodo.title+" to "+remoteTodoRecord.description+"\n");
    if (aLocalRecord.id != aRemoteRecord.id)
      return false;
    if (localTodo.title != remoteTodoRecord.description)
      return false;
    if (localTodo.description != remoteTodoRecord.note)
      return false;
    if (localTodo.priority != ((remoteTodoRecord.priority * 2) - 1))
      return false;
    // XXX due date
dump("same!\n");
    return true;
  },

  // aRemoteRecord is a nsISyncRecord
  // aLocalTodo is a nsILocalTodoSyncRecord
  fillLocalRecord: function (aRemoteRecord, aLocalTodo) {
    var remoteTodo = aRemoteRecord.QueryInterface(Components.interfaces.nsIPalmTodoSyncRecord);
    
    aLocalTodo.title = remoteTodo.description;
    aLocalTodo.description = remoteTodo.note;
    aLocalTodo.priority = (remoteTodo.priority * 2) - 1;

    // Fake a start date. This must be done, otherwise editing the
    // item will fail.
    if (!aLocalTodo.start.getTime()) {
      var now = new Date();
      aLocalTodo.start.setTime(now.getTime());
    }

    if (remoteTodo.indefinite) {
      aLocalTodo.due.clear();
    } else {
      aLocalTodo.due.setTime(remoteTodo.due / 1000);
    }
    
    if (remoteTodo.completed) {
      aLocalTodo.completed.setTime(new Date());
      aLocalTodo.percent = 100;
    } else {
      aLocalTodo.completed.clear();
      aLocalTodo.percent = 0;
    }

    aLocalTodo.syncId = remoteTodo.id;
  },
  
  fillRemoteRecord: function (aLocalRecord, aRemoteRecord) {
    var localTodo = aLocalRecord.QueryInterface(Components.interfaces.nsILocalTodoSyncRecord).localTodo;
    aRemoteRecord.description = localTodo.title;
    aRemoteRecord.note = localTodo.description;
    aRemoteRecord.priority = (localTodo.priority + 1) / 2;

    // XXX fixme
    aRemoteRecord.indefinite = false;
    
    if (localTodo.percent == 100)
      aRemoteRecord.completed = true;
    else
      aRemoteRecord.completed = false;

    aRemoteRecord.id = localTodo.syncId;
  },

  QueryInterface: function (iid) {
    if (!iid.equals(Components.interfaces.nsISyncClient) &&
        !iid.equals(Components.interfaces.nsISupports)) {
        throw Components.results.NS_ERROR_NO_INTERFACE;
    }
    return this;
  },
 
  mPalmConnection : null,
  mCalendar : null,
  mFastPossible : false,
  mLastSync : null,
  mCalendarPath : null
}

var syncModule = {
  firstTime: true,

  registerSelf: function (compMgr, fileSpec, location, type) {
    if (this.firstTime) {
      this.firstTime = false;
      throw Components.results.NS_ERROR_FACTORY_REGISTER_AGAIN;
    }
    compMgr = compMgr.QueryInterface(Components.interfaces.nsIComponentRegistrar);
    compMgr.registerFactoryLocation(this.palmTodoSyncCID,
                                    "Palm Todo Sync Component",
                                    this.palmTodoSyncProgID, 
                                    fileSpec,
                                    location,
                                    type);
  },

  getClassObject: function (compMgr, cid, iid) {
    if (cid.equals(this.palmTodoSyncCID))
      return this.palmTodoSyncFactory;

    if (!iid.equals(Components.interfaces.nsIFactory))
      throw Components.results.NS_ERROR_NOT_IMPLEMENTED;

    throw Components.results.NS_ERROR_NO_INTERFACE;
  },

  palmTodoSyncCID: Components.ID("{c08e986f-0b88-4f63-a6bc-315b9ae0a10b}"),
  palmTodoSyncProgID: "@mozilla.org/sync/todo/palm;1",
  palmTodoSyncFactory: {
    createInstance: function (outer, iid) {
      if (outer != null)
        throw Components.results.NS_ERROR_NO_AGGREGATION;

      var instance = new nsPalmTodoSync();
      instance.init();
      return instance.QueryInterface(iid);
    }
  },

  canUnload: function(compMgr) {
    return true;
  }
};

function NSGetModule(compMgr, fileSpec) {
  return syncModule;
}
